import numpy as np
import matplotlib
matplotlib.use('Agg',force=True)
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

def set_pp(type='paper'):
    # Function:
    # Input: type of plot, default is paper, else is poster; user can implement
    # more types as necessary
    # Output: returns array of colorblind friendly colors
    if type=='paper':
        S_SIZE = 12
        L_SIZE = 14
        mylw=2

        plt.rc('font', size=L_SIZE,weight='bold')          # controls default text sizes
        #plt.rc('axes', titlesize=S_SIZE)     # fontsize of the axes title
        plt.rc('axes', labelsize=L_SIZE,linewidth=mylw)    # fontsize of the x and y labels
        plt.rc('xtick', labelsize=L_SIZE)    # fontsize of the tick labels
        plt.rc('ytick', labelsize=L_SIZE)    # fontsize of the tick labels
        #plt.rc('legend', fontsize=S_SIZE)    # legend fontsize
        #plt.rc('text',usetex=True)
        plt.rcParams['xtick.major.size']=6
        plt.rcParams['xtick.minor.size']=5
        plt.rcParams['ytick.major.size']=6
        plt.rcParams['ytick.minor.size']=5
        plt.rcParams['xtick.major.width']=1.7
        plt.rcParams['xtick.minor.width']=1.7
        plt.rcParams['ytick.major.width']=1.7
        plt.rcParams['ytick.minor.width']=1.7

    else:
        S_SIZE = 28
        L_SIZE = 30
        mylw=4

        plt.rc('font', size=L_SIZE,weight='bold')          # controls default text sizes
        #plt.rc('axes', titlesize=S_SIZE)     # fontsize of the axes title
        plt.rc('axes', labelsize=L_SIZE,linewidth=mylw)    # fontsize of the x and y labels
        plt.rc('xtick', labelsize=L_SIZE)    # fontsize of the tick labels
        plt.rc('ytick', labelsize=L_SIZE)    # fontsize of the tick labels
        #plt.rc('legend', fontsize=S_SIZE)    # legend fontsize
        #plt.rc('text',usetex=True)

        plt.rcParams['xtick.major.size']=12
        plt.rcParams['xtick.minor.size']=10
        plt.rcParams['ytick.major.size']=12
        plt.rcParams['ytick.minor.size']=10
        plt.rcParams['xtick.major.width']=3
        plt.rcParams['xtick.minor.width']=3
        plt.rcParams['ytick.major.width']=3
        plt.rcParams['ytick.minor.width']=3

    safe_col=['#BBBBBB','#0077BB','#33BBEE','#009988','#EE7733','#CC3311','#EE3377']
    other_col = ['k','mediumblue','green','darkorange','purple','cornflowerblue']
    return other_col

def prettyplot(x,y,type='paper',ls='None',marker='o',markersize=5, **kwargs):
    # Function: Create a plot with nice defaults; allows arrays of sizes and colors
    # to be passed, along with multi-dimensional data. Hopefully smart enough to
    # not break when asked something weird
    # Input: x array; y array; purpose of plot (paper or poster, determines sizes
    # and widths), default is paper; linestyle, default is none; marker, default is
    # circles; markersize, default is 5
    # Optional input: s = array of marker sizes, default is all 25 for papertype and
    # all 35 for postertype; color = string or array of colors, if data are multi-
    # dimensional will select smallest dimension to assign different colors to,
    # default is black or different CBF colors defined in set_pp; xlabel = string
    # for xlabel of plot, default is none; ylabel = string for ylabel of plot,
    # default is none; filename = string for name of plot, default is none and no
    # plot will be saved but fig, ax will be returned
    # Output: if no filename, fig, ax will be returned.
    x = np.array(x)
    y = np.array(y)
    if type=='paper':
        myc = set_pp()
        fig,ax = plt.subplots(figsize=(8,6))
        fig.subplots_adjust(left=0.13,bottom=0.11,top=0.97,right=0.97)
        myt = 14
        mys=25
    if type=='poster':
        myc = set_pp(type='poster')
        fig,ax = plt.subplots(figsize=(26,14))
        myt = 30
        mys = 35

    if 's' in kwargs:
        mys = kwargs["s"]
        kwargs = {key: value for key, value in kwargs.items() if key != 's'}

    if 'color' in kwargs:
        mycol = kwargs['color']
        if not isinstance(mycol,list): mycol = [mycol]
        kwargs = {key: value for key, value in kwargs.items() if key != 'color'}
    else:
        if (x.ndim==2): mycol = myc[:min(np.shape(x))]
        else: mycol = myc[0]
        if not isinstance(mycol,list): mycol = [mycol]

    if (len(mycol)>1)&(len(mycol)!=len(x)):
        if (x.ndim==2):
            if (np.shape(x)[0]!=len(mycol))&(np.shape(x)[1]!=len(mycol)):
                print('Warning: length of color array does not match any dimension of data')
                mycol = mycol[:min(np.shape(x))]
        else:
            print('Warning: length of color array does not match any dimension of data')
            mycol = [mycol[0]]

    print(f'x len is {len(x)}, x dim is {x.ndim}, x shape is {np.shape(x)}, and mycol is {mycol}')

    if 'xlabel' in kwargs:
        xlab = kwargs['xlabel']
        kwargs = {key: value for key, value in kwargs.items() if key != 'xlabel'}
    else: xlab = ''
    if 'ylabel' in kwargs:
        ylab = kwargs['ylabel']
        kwargs = {key: value for key, value in kwargs.items() if key != 'ylabel'}
    else: ylab = ''

    if 'filename' in kwargs:
        filename = kwargs['filename']
        kwargs = {key: value for key, value in kwargs.items() if key != 'filename'}
    else: filename=''

    if len(mycol)>1:
        if (len(mycol)==len(x))&(x.ndim==1):
            plt.scatter(x,y,c=mycol,s=mys,marker=marker,**kwargs)
        elif (x.ndim==2):
            if (np.shape(x)[0]==len(mycol)):
                for ii in range(np.shape(x)[0]):
                    plt.scatter(x[ii,:],y[ii,:],color=mycol[ii],s=mys,marker=marker,**kwargs)
            elif (np.shape(x)[1]==len(mycol)):
                for ii in range(np.shape(x)[1]):
                    plt.scatter(x[:,ii],y[:,ii],color=mycol[ii],s=mys,marker=marker,**kwargs)
    else:
        plt.plot(x,y,color = str(mycol[0]),ls=ls,marker=marker,markersize=markersize,**kwargs)

    if len(xlab)>0:
        ax.set_xlabel(xlab,fontsize=myt,weight='bold')

    if len(ylab)>0:
        ax.set_ylabel(ylab,fontsize=myt,weight='bold')

    if len(filename)>0:
        fig.savefig(filename)
        plt.close('all')
        plt.cla()
        plt.clf()
    else:
        return fig,ax

def plotRebound(sm, default=True, **kwargs):
	# Function: plot orbits of rebound simulation
	# Input: rebound simulation; bool for changing default settings (rebound's plotOrbit)
	# Optional inputs: savename = resulting image file name, default is time of creation + .pdf;
	# n_orbits = how many orbits to plot, default is 1; time = how much to integrate the simulation
	# before plotting, default is 0; ntick = how many tick marks on x- and y-axes, default is 5
	from datetime import datetime

	sim = sm.copy()
	mylw=2
	mycol=set_pp()

	if "savename" not in kwargs:
		now = datetime.now()
		mysave = now.strftime("%b-%d-%y-%H:%M:%S")
		kwargs["savename"] = mysave+'.pdf'
	savename = kwargs["savename"]

	if "n_orbits" not in kwargs:
		kwargs["n_orbits"] = 1
	else:
		default = False
	n_orbits = kwargs["n_orbits"]

	if "time" not in kwargs:
		kwargs["time"] = 0 #do not move the simulation forward before plot
	else:
		default = False #time is included, so we need to not use default
	int_time = kwargs["time"]

	if "ntick" not in kwargs:
		kwargs["ntick"] = 5
	ntick = kwargs["ntick"]

	fig,ax = plt.subplots(figsize=(7,7))
	ax.set_aspect("equal")

	ps = sim.particles
	lim = 0
	for pl in ps:
		this_max = max(pl.x,pl.y)
		if this_max > lim: lim = this_max

	fig.subplots_adjust(wspace=0, hspace=0,left=0.25,bottom=0.11,top=0.95)

	if default:
		sim_time=sim.t+int_time
		ax.scatter(ps[0].x, ps[0].y, s=55, marker='*', facecolor='k',zorder=3) #s=35
		ii=0
		for planet in ps[1:]:
			ax.scatter(planet.x, planet.y, s=30, facecolor='k',zorder=3) #s=10
			o = np.array(planet.sample_orbit())
			lc = fading_line(o[:,0],o[:,1],color=mycol[ii],fading=False,lw=2)
			ax.add_collection(lc)
			ii+=1

	else:
		sim_time=sim.t+int_time #from kwargs, 0 if not given
		sim.integrate(sim_time)
		npts = 300*n_orbits
		ps = sim.particles
		numpl = len(ps)-1
		out_P = ps[numpl].P
		maxe = 0
		for ii in range(numpl):
			if ps[ii+1].e > maxe:
				maxe = ps[ii+1].e
		if maxe > 1.0 :
			print('Warning: Planet ejected in integration')

		'''
		if bin_P*n_orbits < 0.1*ps[2].P:
			print("Warning: plotting less than 0.01 of planet orbit")
			print("n_orbits spans %f days" %n_orbits*bin_P)
			print('binary period: %f ; planet period: %f' %(bin_P,ps[2].P))
			print("Scaling n_orbits")
		'''

		time_array = np.linspace(0,out_P*n_orbits,npts)
		xs = np.zeros([npts,numpl])
		ys = np.zeros([npts,numpl])

		for tt in range(len(time_array)):
			sim.integrate(sim_time+time_array[tt])
			for pl in range(numpl):
				xs[tt,pl] = sim.particles[pl+1].x
				ys[tt,pl] = sim.particles[pl+1].y

		ps = sim.particles
		ax.scatter(ps[0].x, ps[0].y, s=55, marker='*', facecolor='k',zorder=3) #s=35

		for pl in range(numpl):
			ax.scatter(ps[pl+1].x, ps[pl+1].y, s=30, facecolor='k',zorder=3) #s=10
			lc = fading_line(xs[:,pl],ys[:,pl],color=mycol[pl],fading=False,lw=2)
			ax.add_collection(lc)
			this_max = max(xs[:,pl])
			if this_max > lim: lim = this_max
			this_max = max(ys[:,pl])
			if this_max > lim: lim = this_max

	lim = 1.25*lim
	ax.set_xlim([-lim, lim])
	ax.set_ylim([-lim, lim])
	ax.set_yticks(np.linspace(-lim,lim,ntick))
	ax.set_xticks(np.linspace(-lim,lim,ntick))
	ax.set_xlabel('AU',weight='bold') #ax.set(ylabel='AU',xlabel='AU')
	ax.set_ylabel('AU',weight='bold')

	pp = PdfPages(savename)
	pp.savefig(fig)
	pp.close()
	plt.close('all')
	plt.cla()
	plt.clf()

def plotBinary(sm, default=True, **kwargs):
	# Function: plot orbits of circumbinary planets
	# Input: rebound simulation; bool for changing default settings (rebound's plotOrbit)
	# Optional inputs: savename = resulting image file name, default is time of creation + .pdf;
	# n_orbits = how many orbits to plot, default is 1; time = how much to integrate the simulation
	# before plotting, default is 0; ntick = how many tick marks on x- and y-axes, default is 7
	from datetime import datetime

	sim = sm.copy()
	S_SIZE = 12
	L_SIZE = 18
	mylw=2

	mycol=set_pp()

	if "savename" not in kwargs:
			now = datetime.now()
			mysave = now.strftime("%b-%d-%y-%H:%M:%S")
			kwargs["savename"] = mysave+'.pdf'
	savename = kwargs["savename"]
	if "n_orbits" not in kwargs:
		kwargs["n_orbits"] = 1
	else:
		default = False
	n_orbits = kwargs["n_orbits"]

	if "time" not in kwargs:
		kwargs["time"] = 0 #do not move the simulation forward before plot
	else:
		default = False #time is included, so we need to not use default
	int_time = kwargs["time"]

	if "ntick" not in kwargs:
		kwargs["ntick"] = 7
	ntick = kwargs["ntick"]

	fig,ax = plt.subplots(figsize=(7,7))
	ax.set_aspect("equal")

	ps = sim.particles
	lim = 0
	for pl in ps:
		this_max = max(pl.x,pl.y)
		if this_max > lim: lim = this_max

	fig.subplots_adjust(wspace=0, hspace=0,left=0.25,bottom=0.11,top=0.95)

	if default:
		sim_time=sim.t+int_time

		for star in ps[:2]:
			ax.scatter(star.x, star.y, s=55, marker='*', facecolor='k',zorder=3) #s=35
		o = np.array(ps[1].sample_orbit())
		lc = fading_line(o[:,0],o[:,1],color=mycol[2],fading=False,lw=2)
		ax.add_collection(lc)


		for planet in ps[2:]:
			ax.scatter(planet.x, planet.y, s=30, facecolor='k',zorder=3) #s=10
			o = np.array(planet.sample_orbit())
			lc = fading_line(o[:,0],o[:,1],color=mycol[4],fading=False,lw=2)
			ax.add_collection(lc)

	else:
		sim_time=sim.t+int_time #from kwargs, 0 if not given
		sim.integrate(int_time)
		npts = 800
		ps = sim.particles
		bin_P = ps[1].P
		if ps[2].e > 1.0:
			print('Planet ejected in integration')
		if bin_P*n_orbits < 0.1*ps[2].P:
			print("Warning: plotting less than 0.01 of planet orbit")
			print("n_orbits spans %f days" %n_orbits*bin_P)
			print('binary period: %f ; planet period: %f' %(bin_P,ps[2].P))
			print("Scaling n_orbits")

		time_array = np.linspace(0,bin_P*n_orbits,npts)
		s1 = np.zeros([npts,2])
		s2 = np.zeros([npts,2])
		pl1 = np.zeros([npts,2])

		for tt in range(len(time_array)):
			sim.integrate(sim_time+time_array[tt])
			s1[tt,0] = sim.particles[0].x
			s1[tt,1] = sim.particles[0].y
			s2[tt,0] = sim.particles[1].x
			s2[tt,1] = sim.particles[1].y
			pl1[tt,0] = sim.particles[2].x
			pl1[tt,1] = sim.particles[2].y

		ps = sim.particles
		for star in ps[:2]:
			ax.scatter(star.x, star.y, s=55, marker='*', facecolor='k',zorder=3) #s=35
		lc1 = fading_line(s1[:,0],s1[:,1],color=mycol[0],fading=False,lw=2)
		ax.add_collection(lc1)
		lc2 = fading_line(s2[:,0],s2[:,1],color=mycol[2],fading=False,lw=2)
		ax.add_collection(lc2)

		for planet in ps[2:]:
			ax.scatter(planet.x, planet.y, s=30, facecolor='k',zorder=3) #s=10
		lc = fading_line(pl1[:,0],pl1[:,1],color=mycol[4],fading=False,lw=2)
		ax.add_collection(lc)

		this_max = max(pl1[:,0])
		if this_max > lim: lim = this_max
		this_max = max(pl1[:,1])
		if this_max > lim: lim = this_max

	lim = 1.25*lim
	ax.set_xlim([-lim, lim])
	ax.set_ylim([-lim, lim])
	ax.set_yticks(np.linspace(-lim,lim,ntick))
	ax.set_xticks(np.linspace(-lim,lim,ntick))
	ax.set_xlabel('AU',weight='bold') #ax.set(ylabel='AU',xlabel='AU')
	ax.set_ylabel('AU',weight='bold')
	#plt.show()

	pp = PdfPages(savename)
	pp.savefig(fig)
	pp.close()
	plt.close('all')
	plt.cla()
	plt.clf()

def planetGif(sm,name, del_dir=True,inverse=False,num=100):
    # Function: creates gif of system through one full orbit of farthest planet
    # Input: rebound simulation of the full system; name of gif; bool to delete
    # the directory and frames for gif, default is True; bool to invert colors
    # so background is black, default is white background; number of frames
    # Output: function does not return anything, but saves gif to wd
    import progressbar
    import os

    if inverse: matplotlib.rcParams['axes.edgecolor'] = 'white'
    print('This might take a while')
    sim = sm.copy()
    sim.move_to_com()
    ps = sim.particles
    numpl = len(ps)-1
    maxP = ps[numpl].P
    mycol = set_prettyplot(type='poster')
    x,y = np.zeros([num,numpl]), np.zeros([num,numpl])
    time = sim.t + np.linspace(0,maxP,num)
    if name[-4:] != '.gif': name +='.gif'

    # create a directory to store the frames; will be deleted after gif is made
    dirn = f'gif-{name[:-4]}'
    if os.path.isdir(dirn) != True:
        os.mkdir(dirn)

    # create the frames
    for ii in range(num):
        sim.integrate(time[ii])
        x[ii,:] = np.array([ps[pl+1].x for pl in range(numpl)])
        y[ii,:] = np.array([ps[pl+1].y for pl in range(numpl)])

    lim = 1.25*max(abs(np.min(x)),np.max(x),abs(np.min(y)),np.max(y))
    widgets = ['Constructing frames: ', progressbar.AnimatedMarker()]
    bar = progressbar.ProgressBar(max_value=num,widgets=widgets).start()
    #bar = progressbar.ProgressBar(max_value=num)
    #widgets=[' [', progressbar.Timer(), '] ', progressbar.Bar(), ' (', progressbar.ETA(), ') ',]

    for ii in range(num):
        if ii%3 == 0:
            fig, ax = plt.subplots(figsize=(14,12))
            fig.subplots_adjust(right=0.96,left=0.23,bottom=0.14,top=0.95)
            fign = f'./{dirn}/{name[:-4]}-{ii:03}.png'  #'{0:03}'.format(1)
            for pl in range(numpl):
                plt.plot(x[:,pl],y[:,pl],color=mycol[pl],lw=8)

            plt.scatter(x[ii,:],y[ii,:],color=mycol[0:numpl],s=400)
            ax.set_xlabel('x [AU]',fontsize=35,weight='bold')
            ax.set_ylabel('y [AU]',fontsize=35,weight='bold')
            ax.axis('equal')
            ax.set(xlim=[-lim, lim], ylim =[-lim, lim])
            ax.set_yticks(np.linspace(-lim,lim,5))
            ax.set_xticks(np.linspace(-lim,lim,3))

            if inverse:
                fig.set_facecolor('black')
                ax.set_facecolor('black')
                ax.tick_params(axis='x', colors='white')
                ax.tick_params(axis='y', colors='white')
                ax.xaxis.label.set_color('white')
                ax.yaxis.label.set_color('white')
            plt.savefig(fign,facecolor=fig.get_facecolor(), edgecolor='none')
            plt.close('all')
            plt.cla()
            plt.clf()
            bar.update(ii)

    # Create the gif
    print('\nCreating gif...')
    print('Sorry there are no fancy animations.....')
    os.system(f'convert -delay 20 -loop 0 ./{dirn}/*.png {name}')
    print(f'gif saved as {name}')

    if del_dir:
        # remove dirn and frames
        print('removing frames')
        os.system(f'rm -fr {dirn}')

def get_hm(xdat,ydat,zdat,x_size,y_size,fig,ax,cm):
    # Function: creates a pretty heatmap
    # Input: array defining the x-axis locations; array defining the y-axis locations;
    # array defining the values for the heatmap; size of x-dim for heatmap; size of
    # y-dim for heatmap; figure handle; axis handle; colormap;
    # Output: returns handle to heatmap
    hm_data = np.zeros([y_size, x_size])
    x_bins, y_bins = np.linspace(min(xdat),max(xdat),(x_size+1)), np.linspace(min(ydat),max(ydat), (y_size+1))
    index = np.array(range(len(xdat)))

    for ii in range(x_size):
    	for jj in range(y_size):
    		these_sims = index[(xdat>x_bins[ii])*(xdat<x_bins[ii+1])*(ydat>y_bins[jj])*(ydat<y_bins[jj+1])]
    		if len(these_sims)>0: hm_data[jj,ii] = np.mean(zdat[these_sims])

    hm = ax.imshow(hm_data,cmap=cm,interpolation='gaussian',aspect='auto',extent=[min(xdat),max(xdat),min(ydat),max(ydat)],origin='lower')

    fig.subplots_adjust(left=0.14,right = 0.85,bottom=0.09,top=0.98)

    return hm

def fading_line(x, y, color='black', alpha=1, fading=True, fancy=False, **kwargs):
    """
    Stolen with love from rebound (Rein & Tamayo 2015 )
    Returns a matplotlib LineCollection connecting the points in the x and y lists.
    Can pass any kwargs you can pass to LineCollection, like linewidgth.
    Parameters
    ----------
    x       : list or array of floats for the positions on the (plot's) x axis.
    y       : list or array of floats for the positions on the (plot's) y axis.
    color   : Color for the line. 3-tuple of RGB values, hex, or string. Default: 'black'.
    alpha   : float, alpha value of the line. Default 1.
    fading  : bool, determines if the line is fading along the orbit.
    fancy   : bool, same as fancy argument in OrbitPlot()
    """
    try:
        from matplotlib.collections import LineCollection
        import numpy as np
    except:
        raise ImportError("Error importing matplotlib and/or numpy. Plotting functions not available. If running from within a jupyter notebook, try calling '%matplotlib inline' beforehand.")


    if "lw" not in kwargs:
        kwargs["lw"] = 1
    lw = kwargs["lw"]

    if fancy:
        kwargs["lw"] = 1*lw
        fl1 = fading_line(x, y, color=color, alpha=alpha, fading=fading, fancy=False, **kwargs)
        kwargs["lw"] = 2*lw
        alpha *= 0.5
        fl2 = fading_line(x, y, color=color, alpha=alpha, fading=fading, fancy=False, **kwargs)
        kwargs["lw"] = 6*lw
        alpha *= 0.5
        fl3 = fading_line(x, y, color=color, alpha=alpha, fading=fading, fancy=False, **kwargs)
        return [fl3,fl2,fl1]

    Npts = len(x)
    if len(y) != Npts:
        raise AttributeError("x and y must have same dimension.")

    color = get_color(color)
    colors = np.zeros((Npts,4))
    colors[:,0:3] = color
    if fading:
        colors[:,3] = alpha*np.linspace(0,1,Npts)
    else:
        colors[:,3] = alpha

    segments = np.zeros((Npts-1,2,2))
    segments[:,0,0] = x[:-1]
    segments[:,0,1] = y[:-1]
    segments[:,1,0] = x[1:]
    segments[:,1,1] = y[1:]

    lc = LineCollection(segments, color=colors, **kwargs)
    return lc



def txtrow_to_sim(row,npl,Mstar,dt,integrator):
	# Funtion: sets up rebound Simulation from row of '{sysName}/{sysName}-allsims.in'
	# Input: system parameters from '{sysName}/{sysName}-allsims.in' for sim ID;
	# number of planets; mass of star; integration step size; integrator
	# Output: returns rebound Simulation to integrate
	import rebound
	import numpy as np

	sim=rebound.Simulation()
	sim.units=('Msun','days','AU') #hard coded for now
	#sim.units=(m_un,t_un,d_un)
	#Mstar = row['Mstar']
	sim.add(m=Mstar)
	for ii in range(1,npl+1):
	   m = row['m{}'.format(ii)]*3.0e-6 #convert from M_Earth to M_Sun
	   per = row['p{}'.format(ii)]
	   inc = (90-row['inc{}'.format(ii)])*np.pi/180.0
	   ecc = row['ecc{}'.format(ii)]
	   Omega = row['Om{}'.format(ii)]
	   mean_long = row['mLon{}'.format(ii)]
	   #pomega = row['pomega{}'.format(ii)]
	   sim.add(m=m,P=per,inc=inc,e=ecc,Omega=Omega,l=mean_long,hash=ii)
	sim.move_to_com()
	sim.integrator=integrator
	minP = min([sim.particles[x+1].P for x in range(npl)])
	sim.dt = minP*dt
	set_min_distance(sim,2)
	return sim

def set_min_distance(sim,rHillFactor):
    # Function: sets minimum distance allowed for rebound
    # Input: rebound simulation, number of hill radii as min
    # Output: None
    import numpy as np

    ps=sim.particles[1:]
    rHill = np.min([p.rhill for p in ps if p.m > 0])
    minDist = rHillFactor*rHill
    sim.exit_min_distance=minDist

def drawmass(value, mass=False, **kwargs):
    # Function: draws mass from normal distribution
    # Input: either mass or radius; bool for if passing mass or radius, default is
    # radius
    # Optional input: err =  uncertainty on mass distribution, default is 30%
    # Output: returns mass
    if mass: #value is the mass
        if "err" not in kwargs:
            kwargs["err"] = 0.3*value
        m_val = drawnormal(value,kwargs["err"])

    else: #value is the radius
        draw_mass = mr(value) #returns a radius
        if "err" not in kwargs:
            kwargs["err"] = 0.3*draw_mass
        m_val = drawnormal(draw_mass,kwargs["err"])
    return m_val

def mr(value, mass=False, radius=True):
    # Function: returns mass or radius from M-R relation (Bashi et al 2017)
    # Input: mass or radius; bool for passing mass, default is False; bool
    # Output: returns the radius or mass

    if mass: #return a radius for some reason
        if value < 124.0: return value**0.55
        else: return value**0.01

    if radius: #return a mass
        if value < 12.1: return value**(1.0/0.55)
        else: return value**100.0

def resProx(p1,p2,MMR):
    # Function: returns proximity to first-order resonance
    # Input: orbital period 1, orbital period 2, order
    # Output: P2/P1 - (MMR+1)/MMR  where MMR = 1 is for 2:1, 2 is 3:2, 3 is 4:3 etc

    if p1>p2:
        dummy = p1
        p1 = p2
        p2 = dummy
    prox = (p2/p1) - ((MMR+1)/MMR)
    return prox

def nearestRes(p):
    # Function: returns coefficients for two- and three-body resonant angles, based
    # on period ratios
    # Input: array of orbital periods
    # Output: returns coefficients of two- and three-body resonant angles; returns
    # array of zeros if period ratios are not near any strong resonances
    import numpy as np

    npl = len(p)
    n2br = npl-1
    n3br = npl-2
    res = np.zeros([(n2br+n3br),3])
    for i in range(n2br):
        this_ratio = p[i+1]/p[i]
        if this_ratio < 3.2 and this_ratio > 2.98 :
            res[i,:] = np.array([3,-1,-2])
        elif this_ratio < 2.7 and this_ratio > 2.48 :
            res[i,:] = np.array([5,-2,-3])
        elif this_ratio < 2.2 and this_ratio > 1.88 :
            res[i,:] = np.array([2,-1,-1])
        elif this_ratio < 1.73 and this_ratio > 1.66 :
            res[i,:] = np.array([5,-3,-2])
        elif this_ratio < 1.6 and this_ratio > 1.48 :
            res[i,:] = np.array([3,-2,-1])
        elif this_ratio < 1.48 and this_ratio > 1.3 :
            res[i,:] = np.array([4,-3,-1])
        elif this_ratio < 1.3 and this_ratio > 1.2 :
            res[i,:] = np.array([5,-4,-1])
        else: res[i,:] = np.array([0,0,0])

    for i in range(n3br):
        order= -1.0*np.array([res[i,2],res[i+1,2]])
        c1 = res[i+1,0]*order[0]
        c2 = res[i+1,1]*order[0]-res[i,0]*order[1]
        c3 = -1.0*res[i,1]*order[1]
        if np.mod(abs(c1),2) == 0 and np.mod(abs(c2),2) == 0 and np.mod(abs(c3),2) == 0:
            c1 = c1/2; c2 = c2/2; c3 = c3/2
        if np.mod(abs(c1),3) == 0 and np.mod(abs(c2),3) == 0 and np.mod(abs(c3),3) == 0:
            c1 = c1/3; c2 = c2/3; c3 = c3/3

        res[i+n2br,:] = [c1,c2,c3]
    if sum(sum(res)): print('Warning: no resonances likely')
    return res

def mad(array):
    # Function: calculates and returns the MAD of an array
    # Input: array of numerical values
    # Output: returns MAD
    import numpy as np

    this_med = np.median(array)
    return np.median([abs(a-this_med) for a in array])

def wrap(vals):
    # Function: wraps an angle between 0 and 360 degress
    # Input: array of values, in radians
    # Output: array of values, wrapped, in degrees
    import numpy as np

    for ii in range(len(vals)):
      while vals[ii] < 0.0:
      	vals[ii] += 2.0*np.pi
      while vals[ii] > 2.0*np.pi:
      	vals[ii] -=2.0*np.pi
    return vals*180.0/np.pi

def wrap180(vals):
    # Function: wraps an angles between -180 and 180 degrees
    # Input: array of values, in radians
    # Output: array of values, wrapped, in degrees
    import numpy as np

    for ii in range(len(vals)):
      while vals[ii] < -1.0*np.pi:
      	vals[ii] += 2.0*np.pi
      while vals[ii] > np.pi:
      	vals[ii] -=2.0*np.pi
    return vals*180.0/np.pi

def drawnormal(mean, sigma):
	# Function: draws value from a normal distribution
	# Input: mean and standard deviation; can be arrays
	# Output: returns draw from normal distribution, value must be positive
	from numpy.random import normal
	import numpy as np

	vals = np.zeros(len(mean))
	for ii in range(len(mean)):
		val = -1
		while val < 0:
			val = normal(mean[ii], sigma[ii])
		vals[ii] = val
	return list(vals)

def getCol(numpl):
    # Function: returns the column numbers required for characterizing resonances
    # called in calc_res() and constrainProp()
    # Input: number of planets
    # Output: returns array of column indices
    col={}
    if numpl == 3:
        col[0] = [1,3]
        col[1] = [1, 2, 3]
        col[2] = [2, 3]
    else:
        for pl in range(numpl):
            if pl==0: col[pl] = [1,numpl]
            elif pl==1: col[pl] = [1,2,numpl,numpl+1]
            elif pl==(numpl-2): col[pl] = [pl,pl+1, numpl-2+pl,numpl-1+pl]
            elif pl==(numpl-1): col[pl] = [pl,numpl-2+pl]
            else: col[pl] = [pl,pl+1,numpl-2+pl,numpl-1+pl,numpl+pl]
    return col

def clean_up(sysName, bin=0, res=1, job=1,submit=1):
	# Function: cleans up directories
	# Input: system name; bool for removing simulationArchive bin files, default is
	# False; bool for removing res amp/cen files, default is True so make sure to
	# run combine_res() first; bool for removing SLURM job files, default is
	# True; bool for removing submit files, default is True
	# Output: None returned, files deleted
	import os
	import os.path

	if bin:
		os.system(f'rm ./{sysName}/bin_files/*.bin')

	if res:
		if os.path.exists(f'./{sysName}/{sysName}_res_cen.dat'):
			os.system(f'rm ./{sysName}/res_cen/*.txt')

	if job:
		os.system(f'rm ./{sysName}/job_files/job*.out')

	if submit:
		os.system(f'rm ./{sysName}/job_files/*.sh')

def maxe(ps,numpl):
	# Function: exits python if any planet's ecc is greater than 1 (unbound)
	# Input: sim.particles of rebound Simulation; number of planets
	# Output: None returned
	import numpy as np

	maxecc = max(np.array([ps[pl+1].e for pl in range(numpl)]))
	if maxecc > 1.0 :
		print('Warning: Planet ejected in integration')
		return 1

def stats(x):
	# Function: calculates and returns statistics of an array
	# Input: array of values
	# Output: returns the median, upper uncertainty, and lower uncertainty of array
	import numpy as np
	m = np.median(x)
	u = np.percentile(x,84)-m
	d = m-np.percentile(x,16)
	return m,u,d

def compare(a,res):
	# Function: compares a parameter, split by a binary array
	# Input: distribution of one parameter; array of bool to separate populations
	# Output: None returned, but prints to screen the median +\- uncertainties of
	# each population as well as the p-val from a two-sample K-S test
	from scipy.stats import ks_2samp

	if((sum(res)==len(a)) | (sum(res)==0)):
		return
	st, pval = ks_2samp(a[(res==1)],a[(res==0)])
	if pval < 0.05:
		a1,a2,a3 = stats(a[(res==1)])
		b1,b2,b3 = stats(a[(res==0)])
		print(f'res: {a1:.2f}+{a2:.2f}-{a3:.2f}')
		print(f'non-res: {b1:.2f}+{b2:.2f}-{b3:.2f}')
		print(f'k-s p-vale = {pval}')
		print('**********************************')



import numpy as np

def read_data(filename):
    # Function: Sorts info from input file into different arrays
    # Input: name of input file (created with exoMMR.make_input())
    # Output: returns three arrays to be passed to make_sim_file()
    import pandas as pd

    c = 0
    for line in open(filename):
        li = line.strip()
        if not li.startswith('#'):
            if c==1:
                numpl = int(line.rstrip())
                break
            elif c==0: c=1

    col_heads=[*(f'pl{ii}' for ii in range(1, (numpl+1)))]
    data = pd.read_csv(filename,sep='\t',header=None,comment = '#',names = col_heads)
    sys_name = data.iloc[0,0] #string
    numpl = int(data.iloc[1,0]) #int
    m_s = float(data.iloc[2,0]) #float
    units = str(data.iloc[3,0:3]) #this keeps the keys for some reason
    units = ['days','Msun','AU']
    pl_data = np.array(data.iloc[4:14,:], dtype = float) #2D array, float
    numSim = int(data.iloc[14,0]) #int
    int_time = float(data.iloc[15,0]) #float
    dt = float(data.iloc[16,0]) #float
    savebin = int(data.iloc[17,0]) #bool
    integrator = data.iloc[18,0] #string
    archive_step = float(data.iloc[19,0]) #float

    sim_par_str = [sys_name,integrator,units]
    sim_par = np.array([numpl,savebin,m_s,numSim,int_time,dt,archive_step])

    return sim_par_str, sim_par, pl_data

def make_sim_file(dir, sim_par_str, sim_par, pl_data):
    # Function: creates two files, one with all system parameters for all simulations
    # and another with all rebound parameters
    # Input: path of system directory; three arrays from read_data
    # Output: None returned, but two files saved as '{sysName}/{sysName}-allsims.in'
    # and '{sysName}/{sysName}-rb.param'
    from astropy.table import Table
    from numpy.random import seed, uniform
    import rebound
    from .tools import drawnormal
    from os.path import exists

    filename = dir+'/'+sim_par_str[0]+'-allsims.in'
    filename2 = dir+'/'+sim_par_str[0]+'-rb.param'

    if exists(dir+'/../slurm.in') != True:
        fs = open(dir+'/../slurm.in','w')
        fs.write('#SBATCH --ntasks=1\n')
        fs.write('#SBATCH --nodes=1\n')
        fs.write('#SBATCH --partition=normal\n')
        fs.write('#SBATCH --time=4:00:00\n')
        fs.close()

    #7/29: do I add units to row 1?
    numpl = int(sim_par[0])
    nCol = 1+numpl*6
    nRow = int(sim_par[3]) #numSim
    data = np.zeros([nRow,nCol])

    for ii in range(nRow):
    	seed(ii)
    	m = drawnormal(pl_data[0,:],pl_data[1,:])
    	p = drawnormal(pl_data[2,:],pl_data[3,:])
    	inc = drawnormal(pl_data[4,:],pl_data[5,:])
    	ecc = drawnormal(pl_data[6,:],pl_data[7,:])
    	ecc = [0.0 if x < 0 else x for x in ecc]
    	Om = list(uniform(0, 2.0*np.pi,numpl))
    	mLon = list(np.mod(-2.0*np.pi+pl_data[8,:]/p,2.0*np.pi))
    	data[ii,:] = np.array([ii]+m+p+inc+ecc+Om+mLon)

    col_heads=['ind',*(f'm{ii}' for ii in range(1, (numpl+1))),*(f'p{ii}' for ii in range(1, (numpl+1))),*(f'inc{ii}' for ii in range(1, (numpl+1))),*(f'ecc{ii}' for ii in range(1, (numpl+1))),*(f'Om{ii}' for ii in range(1, (numpl+1))),*(f'mLon{ii}' for ii in range(1, (numpl+1)))]
    wdata = Table(data,names=col_heads)
    wdata.write(filename, format='ascii', overwrite=True)

    f = open(filename2,'w')
    for t in sim_par_str: f.write('{}\t'.format(t))
    for t in sim_par: f.write('{}\t'.format(str(t)))
    f.close()

def sim(ind, sysName, track_res = True, Nout = 2000):
	import numpy as np
	from astropy.table import Table
	import rebound
	import sys
	import pandas as pd
	from .tools import txtrow_to_sim
	M_earth = 3.e-6
	toearth = 3.33e5

	pldatfile = './'+sysName+'/'+sysName+'-allsims.in'
	simparamfile = './'+sysName+'/'+sysName+'-rb.param'

	pl_data = Table.read(pldatfile,format='ascii',fast_reader=False)
	sim_param = pd.read_csv(simparamfile,sep='\t',header=None)
	sim_param = sim_param.iloc[0]
	units_arr = sim_param[2] #('days', 'AU', 'Msun')
	sim_integrator = sim_param[1] #'whfast'
	T1 = float(sim_param[7])*365.25
	dt = float(sim_param[8])
	myint = float(sim_param[9]) #interval for rebound to save points; ignored if savebin==0
	M_star = float(sim_param[5])
	numpl = int(sim_param[3])
	archive = int(sim_param[4])

	sim = txtrow_to_sim(pl_data[ind],numpl,M_star,dt,sim_integrator)
	binname = './bin_files/%s-%i.bin' %(sysName,ind)
	if archive: sim.automateSimulationArchive(binname,interval=myint,deletefile=True)

	try:
		sim.integrate(T1)

	except rebound.Encounter as error:
		print(error)
		ps = sim.particles
		print(f'time={sim.t}, dt = {sim.dt}')
		print(f'P=[{ps[1].P},{ps[2].P},{ps[3].P},{ps[4].P}]')
		print(f'e=[{ps[1].e},{ps[2].e},{ps[3].e},{ps[4].e}]')
		print(f'mass=[{ps[1].m*toearth},{ps[2].m*toearth},{ps[3].m*toearth},{ps[4].m*toearth}]')
		exit()

	endsim = f'./{sysName}/bin_files/end{ind}.bin'
	sim.save(endsim)

	make_rescen(sysName,endsim,incb=False)
	print("integration complete")

def run_sim(num,sysName,wd):
	# Function: submits SLURM job to run rebound simulation
	# Input: simulation number/ID, system name, working directory
	# Output: None returned, but creates and executes SLURM file
	import os

	sysDir = wd+'/'+sysName
	jobDir = '/job_files'
	fn = sysDir+'/'+sysName+'-allsims.in' #wd/sysName/sysName-allsims.in
	fn2 = sysDir+'/'+sysName+'-rb.param'
	fn3 = wd+'/'+'slurm.in'

	f3 = open(fn3,"r")
	lines = f3.readlines()
	f3.close()

	os.system('echo %i' % num)
	f=open('%s%s/submit.sh' %(sysDir,jobDir),'w')
	f.write('#!/bin/bash \n')
	f.write('\n')
	f.write('#SBATCH --chdir=%s/ \n' %wd)
	f.write('#SBATCH --job-name=res%a \n')
	f.write(f'#SBATCH --output={sysDir}{jobDir}/job.%a.out  \n')
	f.write('#SBATCH --array=0-%i \n' %(num))
	for l in lines:
		f.write(l)
	f.write('\n')
	f.write('module load python \n')
	f.write('\n')
	f.write('echo \"Starting on \"`date` \n')
	f.write('python -c \"import exoMMR; exoMMR.sim($SLURM_ARRAY_TASK_ID,\'%s\')\" \n' %(sysName))

	f.write('echo \"Finished on \"`date` \n')
	f.close()

	cmdstr = 'sbatch %s%s/submit.sh' %(sysDir,jobDir)
	os.system(cmdstr)

def combine_res(sysName):
	# Function: combines amp/cen res files from all simulations into one file;
	# must be proceeded by either track_res = True in exoMMR.sim() or by exoMMR.make_rescen()
	# Input: system name
	# Output: None returned, but creates file './{sysName}/{sysName}_res_cen.dat'
	import os

	comb_file = f'./{sysName}/{sysName}_res_cen.dat'
	f = open(comb_file,'w')
	ii=0

	for file in os.listdir(f'./{sysName}/res_cen/'):
		if file.endswith(".txt"):
			#read file and put stuff into other doc
			ii=ii+1
			if ii%25==0: print(ii)
			newfile = file
			f1 = open(f'./{sysName}/res_cen/'+newfile,'r')
			if ii==1:
				temp_string = f1.read()
				f.write(temp_string)
				f.write('\n')
			else:
				temp_string = f1.readlines()
				f.write(temp_string[2])
				f.write('\n')
			f1.close()
	f.close()

def calc_res(sysName):
    # Function: performs statistics on number of simulations with librating angles
    # must be proceeded by combine_res()
    # Input: system name
    # Output: None returned, but creates two files.
    # './{sysName}/{sysName}_librating.dat' contains flags for which angle librated
    # in which simulation; './{sysName}/{sysName}_stats.dat' contains centers and
    # amplitudes for librating angles
    from astropy.table import Table

    comb_file = f'./{sysName}/{sysName}_res_cen.dat'
    lib_file = f'./{sysName}/{sysName}_librating.dat'
    stats_file = f'./{sysName}/{sysName}_stats.dat'

    data = Table.read(comb_file,format='ascii')
    names = data.colnames
    a =  len(data)#a is rows, b is columns
    b = len(names)
    num_angles = int((b-1)/2)
    names = np.array([x[:-2] for x in names])
    name = names[np.array([((ii*2)+1) for ii in range(num_angles)])]

    amp_max = 150.0

    lib = np.zeros([a,(num_angles+1)])
    lib[:,0] = np.array(data[0][:])

    f1 = open(stats_file,'w')
    for ii in range(num_angles):
        this_cen = data.columns[((ii*2)+1)]
        this_amp = data.columns[((ii*2)+2)]
        this_amp = np.array([200 if x==0 else x for x in this_amp])
        lib[:,(ii+1)] = np.array([1 if x<amp_max else 0 for x in this_amp])
        try:
            c1,c2,c3 = np.percentile(this_cen[this_amp<amp_max],[16,50,84])
            a1,a2,a3 = np.percentile(this_amp[this_amp<amp_max],[16,50,84])
            f1.write(f'{name[ii]}_c\t{c2}\t{c3-c2}\t{c2-c1}\t{(sum(lib[:,(ii+1)])/a)*100.0:.2f}%\n')
            f1.write(f'{name[ii]}_a\t{a2}\t{a3-a2}\t{a2-a1}\t{(sum(lib[:,(ii+1)])/a)*100.0:.2f}%\n')
        except:
            f1.write(f'{name[ii]}_c\tNone\tNone\tNone\t0.00%\n')
            f1.write(f'{name[ii]}_a\tNone\tNone\tNone\t0.00%\n')
    f1.close()

    wdata = Table(lib,names=np.append(['index'],name))
    wdata.write(lib_file, format='ascii', overwrite=True)

def char_res(sysName,incb=False):
	# Function: characterizes the resonance chain(s) in system. Must be proceeded
	# by calc_res(); a chain requires 3+ planets
	# Input: system name; bool for if both two-body angle are included in *librating.dat
	# Output: None returned, but creates './{sysName}/{sysName}_char-res.dat' with
	# characterization information
	from itertools import groupby
	from astropy.table import Table
	from .tools import getCol

	lib_file = f'./{sysName}/{sysName}_librating.dat'
	char_file = f'./{sysName}/{sysName}_char-res.dat'

	data = Table.read(lib_file, format = 'ascii')
	aDat = np.lib.recfunctions.structured_to_unstructured(data.as_array())
	N,b = np.shape(aDat)
	index = aDat[0,:]
	numpl = int((b+2)/2)
	if incb: numpl = int((b+3)/3)
	if numpl<3:
		print("This function requires at least three planets")
		exit()

	part_chain = np.ones([N,numpl])*300 #is the planet part of the chain in this simulation
	col = getCol(numpl)

	if incb:
		for ii in range(numpl-1):
			aDat[:,1+ii] = [max(aDat[q,1+ii], aDat[q,2*numpl-2+ii] ) for q in range(N)]

	for pl in range(numpl):
		part_chain[:,pl] = [1 if(sum(aDat[x,col[pl]])>0) else 0 for x in range(N)]

	max_chain = np.zeros(N)

	for x in range(N):
		if sum(part_chain[x,:])>1:
			max_chain[x] = max(len(list(g)) for k,g in groupby(part_chain[x,:]))

	for ii,m in enumerate(max_chain):
		if m == numpl:
			if (sum(aDat[ii,1:numpl]) < (numpl-1)) & (sum(aDat[ii,numpl:])==0): #if not all two-body angles librate AND no three-body angles librate
				max_chain[ii] = 2

	f = open(char_file,'w')
	for pl in range(numpl):
		f.write(f'Percentage of stable simulations where planet {pl+1} is in res chain: {100*sum(part_chain[:,pl])/N}%\n')
	for c in range(1,numpl):
		f.write(f'Percentage of stable simulations with at least a {c+1}-body resonance: {100*len(max_chain[(max_chain>(c))])/N }%\n')

	f.close()

	all_data = np.zeros([N,(b+1+numpl)])
	header = data.colnames
	header += [f'pl{pl+1}' for pl in range(numpl)]
	header += ['len_chain']

	all_data[:,:b] = aDat
	all_data[:,b:(b+numpl)] = part_chain
	all_data[:,-1] = max_chain

	wdata = Table(all_data,names=header)
	allfile = f'./{sysName}/{sysName}_all.dat'
	wdata.write(allfile, format='ascii', overwrite=True)

def make_input(datfile,KOI,col_name='KOI'):
	# Function: creates the input file for read_data() from a csv; user may also
	# update an example input file by hand
	# Input: system number, either KOI or TOI
	# Output: None returned, but creates '{sysName}.in' for read_data()
	import pandas as pd
	from .tools import mr

	dat = pd.read_csv(datfile,sep=',')
	nrows = dat.shape[0]
	rows = np.array([int(x) for x in np.linspace(0,nrows,nrows)])
	all_KOI = np.array(dat[col_name].values.tolist())
	myrows = rows[(all_KOI==KOI)]

	sysName = f'{col_name}-{KOI}'
	numpl = len(myrows)

	mass = np.array(dat['mass'][myrows])
	m_err = np.array(dat['m_err'][myrows])
	period = np.array(dat['period'][myrows])
	p_err = np.array(dat['p_err'][myrows])
	inc = np.array(dat['inc'][myrows])
	i_err = np.array(dat['i_err'][myrows])
	ecc = np.array(dat['ecc'][myrows])
	e_err = np.array(dat['e_err'][myrows])
	t0 = np.array(dat['t0'][myrows])
	t0_err = np.array(dat['t0_err'][myrows])
	radius = np.array(dat['radius'][myrows])
	r_err = np.array(dat['r_err'][myrows])
	Mstar = np.array(dat['st_mass'][myrows])[0]

	sort = np.argsort(period)
	mass = mass[sort]
	m_err = m_err[sort]
	period = period[sort]
	p_err = p_err[sort]
	inc = inc[sort]
	i_err = i_err[sort]
	ecc = ecc[sort]
	e_err = e_err[sort]
	t0 = t0[sort]
	t0_err = t0_err[sort]
	radius = radius[sort]
	r_err = r_err[sort]

	for m in range(numpl):
		if np.isnan(mass[m]):
			mass[m]=mr(radius[m],radius=True)
			m_err[m]=mass[m]*0.3

	'''
	# KOI host mass in separate file
	dat2 = pd.read_csv('./data/mstar_koi.csv',sep=',') #KOI,Mstar
	mstars = np.array(dat2['Mstar'])
	mKOI = np.array(dat2['KOI'])
	Mstar = float(mstars[(mKOI==KOI)])
	'''

	num = 500
	T1 = 1e6
	tstep = 0.05
	savebin = 0
	integrator = 'whfast'
	arch_step = 1e4

	filename = f'{sysName}.in'
	f = open(filename,'w')
	f.write( '# System name\n{}\n# Number of planets\n{}\n# Stellar mass\n{}\n# Units\ndays\tMsun\tAU\n# Planet masses\n'.format(sysName,numpl,Mstar) )
	for x in mass: f.write('{}\t'.format(x))
	f.write('\n# Mass error\n')
	for x in m_err: f.write('{}\t'.format(x))
	f.write('\n# Periods\n')
	for x in period: f.write('{}\t'.format(x))
	f.write('\n# Period error\n')
	for x in p_err: f.write('{}\t'.format(x))
	f.write('\n# Inc\n')
	for x in inc: f.write('{}\t'.format(x))
	f.write('\n# Inc error\n')
	for x in i_err: f.write('{}\t'.format(x))
	f.write('\n# Ecc\n')
	for x in ecc: f.write('{}\t'.format(x))
	f.write('\n# Ecc error\n')
	for x in e_err: f.write('{}\t'.format(x))
	f.write('\n# t0\n')
	for x in t0: f.write('{}\t'.format(x))
	f.write('\n# t0 error #unused\n')
	for x in t0_err: f.write('{}\t'.format(x))
	f.write('\n# Number of simulations\n{}\n# Integration time\n{}\n# Time step percentage\n{}\n'.format(num,T1,tstep))
	f.write('# Flag for saving bin file\n{}\n'.format(savebin))
	f.write('# Rarely changed parameters:\n# Integrator\n{}\n# Sim-archive step\n{}\n'.format(integrator,arch_step))
	f.close()

def make_rescen(sysName,binname,incb=False):
	# Function: creates res file with centers and amplitudes for a simulation
	# Input: system name; string for the simulationArchive bin file; bool to include
	# additional two-body MMR (other pomega)
	# Output: None returned, but creates './{sysName}/res_cen/{sysName}_res_{ind}.txt'
	from re import findall
	import rebound
	from .tools import maxe, nearestRes, wrap, wrap180

	ind = int(findall(r'[+]?[\d]+(?:,\d\d\d)*(?:[eE][-+]?\d+)?',binname)[-1])
	try:
		sm = rebound.Simulation(binname)
	except:
		print(f'file {binname} does not exist')
		return
	sim = sm.copy()
	sim_time=sim.t

	Nout=2000
	ps = sim.particles
	numpl = len(ps)-1
	check = maxe(ps,numpl)
	if check == 1:
		return
	l = np.zeros([numpl,Nout]); po = np.zeros([numpl, Nout]); p = np.zeros([numpl, Nout]); e = np.zeros([numpl, Nout])
	int_time = np.linspace(0,6000,Nout)

	for ii,t in enumerate(int_time):
		sim.integrate(sim_time+t)
		for pl in range(numpl):
			l[pl,ii] = sim.particles[pl+1].l
			po[pl,ii] = sim.particles[pl+1].pomega
			p[pl,ii] = sim.particles[pl+1].P
			e[pl,ii] = sim.particles[pl+1].e
	ps = sim.particles
	check = maxe(ps,numpl)
	if check ==1:
		return

	allper = np.array([ps[pl+1].P for pl in range(numpl)])
	res_ang = np.zeros([(numpl*2-3),Nout])
	if incb: phi_bs = np.zeros([(numpl-1),Nout])
	thbr_names = ['lap1','lap2','lap3','lap4','lap5']
	twbr_names = ['phi12','phi23','phi34','phi45','phi56','phi67']
	ang_names = []

	res = nearestRes(allper)

	for kk in range(numpl-1):
		phi = res[kk,0]*l[(kk+1),:] + res[kk,1]*l[kk,:] + res[kk,2]*po[kk,:]
		phib = res[kk,0]*l[(kk+1),:] + res[kk,1]*l[kk,:] + res[kk,2]*po[(kk+1),:]
		aa = wrap180(phi)
		bb = wrap(phi)
		if 2.0*np.std(aa) < 2.0*np.std(bb):
			res_ang[kk,:] = aa
		else:
			res_ang[kk,:] = bb
		if incb:
			phi_bs[kk,:] = wrap180(phib)
			aa = wrap180(phib)
			bb = wrap(phib)
			if 2.0*np.std(aa) < 2.0*np.std(bb):
				phi_bs[kk,:] = aa
			else:
				phi_bs[kk,:] = bb

		c_name = twbr_names[kk]+'_c'
		a_name = twbr_names[kk]+'_a'
		ang_names = ang_names + [c_name,a_name]
	for kk in range(numpl-2):
		lap = res[kk+numpl-1,0]*l[(kk+2),:] + res[kk+numpl-1,1]*l[(kk+1),:] + res[kk+numpl-1,2]*l[kk,:]
		aa = wrap(lap)
		bb = wrap180(lap)
		if 2.0*np.std(aa) < 2.0*np.std(bb):
		  res_ang[(kk+numpl-1),:] = aa
		else:
		  res_ang[(kk+numpl-1),:] = bb
		c_name = thbr_names[kk]+'_c'
		a_name = thbr_names[kk]+'_a'
		ang_names = ang_names + [c_name,a_name]

	txtfile = f'./{sysName}/res_cen/{sysName}_res_{ind}.txt'

	f = open(txtfile,'w')
	f.write("# index\t")
	for ii in range(len(ang_names)):
		f.write(ang_names[ii]+'\t')
	if incb: f.write('phi12b_c\tphi12b_a\tphi23b_c\tphi23b_a\n')
	else: f.write('\n')
	f.write("# angles = ")
	for ii in range(len(res_ang)):
		f.write(f'[{res[ii,0]},{res[ii,1]},{res[ii,2]}]\t')

	f.write('\n'+str(ind))

	for ii in range(len(res_ang)):
		this_ang = res_ang[ii,:]
		center = np.median(this_ang)
		amp = 2.0*np.std(this_ang)
		if amp == 0: amp = 200.0
		f.write('\t'+str(center)+'\t'+str(amp))

	if incb:
		for ii in range(numpl-1):
			this_ang = phi_bs[ii,:]
			center = np.median(this_ang)
			amp = 2.0*np.std(this_ang)
			if amp==0: amp = 200.0
			f.write('\t'+str(center)+'\t'+str(amp))
	f.close()

def constrain_prop(sysName):
	# Function: calls exoMMR.compare() to constrain mass of each planet
	# based on librating angles
	# Input: system name; must be proceeded by calc_res()
	# Output: None returned
	from .tools import getCol, compare

	filename1 = f'./{sysName}/{sysName}-allsims.in'
	filename2 = f'./{sysName}/{sysName}_librating.dat'
	with open(filename2) as f:
		ang_names = np.array(f.readline().rstrip().split(' '))

	dat1 = np.loadtxt(filename1,skiprows=1)
	dat2 = np.loadtxt(filename2,skiprows=1)
	numpl = int((np.shape(dat1)[1]-1)/6)

	index = dat2[:,0]
	index = np.array([int(x) for x in index])
	col = getCol(numpl)
	for pl in range(numpl):
		mass = dat1[index,(pl+1)]
		ang = dat2[:,col[pl]]
		anames = ang_names[col[pl]]
		for ii in range(len(col[pl])):
			print(f'planet {pl+1}: {anames[ii]}')
			compare(mass,ang[:,ii])



from setuptools import setup

setup(
    name='exoMMR',
    version='1.1.2',
    description='A package focused on confirming and characterizing mean motion resonance in exoplanetary systems',
    url='https://github.com/gracefulmacdonald/exoMMR',
    author='Mariah MacDonald',
    author_email='mariah.g.macdonald@gmail.com',
    license='GPL',
    packages=['exoMMR'],
    install_requires=['matplotlib',
                      'numpy',
                      'rebound',
                      'pandas',
                      'astropy',
                      'scipy',
                      'progressbar',
                      ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
    ],
)



#
from .tools import txtrow_to_sim, nearestRes, clean_up, compare
from .analysis import read_data, make_sim_file, run_sim, sim, combine_res, calc_res, char_res, make_input, constrain_prop, make_rescen
from .plotRes import plotRes

__all__ = ['sim','txtrow_to_sim', 'nearestRes', 'clean_up', 'compare','read_data', 'make_sim_file', 'run_sim', 'combine_res', 'calc_res', 'char_res', 'make_input', 'constrain_prop','make_rescen']



import numpy as np
import rebound
from datetime import datetime
import matplotlib
matplotlib.use('Agg',force=True)
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from .tools import nearestRes,wrap180,wrap
from .myplot import set_pp

def plotRes(sm, addlines=True,incb=0,**kwargs):
    # Function: plots the resonant angles of simulation
    # Input: rebound simulation; bool for adding lines at center and amp of angles; bool for including
	# alternative two-body angles (with other planet's pomega)
    # Optional inputs: savename = resulting image file name, default is time of creation + .pdf;
    # time = how much to integrate the simulation before plotting, default is 0;
    # ntick = how many tick marks on x- and y-axes, default is 5
    # Output: None
	sim = sm.copy()
	mylw=2
	mycol = set_pp()

	if "savename" not in kwargs:
		now = datetime.now()
		mysave = now.strftime("%b-%d-%y-%H:%M:%S")
		kwargs["savename"] = mysave+'.pdf'
	savename = kwargs["savename"]

	if "time" not in kwargs:
		kwargs["time"] = 0.0 #do not move the simulation forward before plot
	int_time = kwargs["time"]

	if "ntick" not in kwargs:
		kwargs["ntick"] = 5
	ntick = kwargs["ntick"]

	sim_time=sim.t+int_time #from kwargs, 0 if not given
	sim.integrate(sim_time)
	Nout=2000
	ps = sim.particles
	numpl = len(ps)-1
	if numpl < 3: print('Warning: simulation contains fewer than 3 planets')

	maxe(ps,numpl)

	l = np.zeros([numpl,Nout]); po = np.zeros([numpl, Nout]); p = np.zeros([numpl, Nout]); e = np.zeros([numpl, Nout])
	int_time = np.linspace(0,6000,Nout)

	for ii,t in enumerate(int_time):
		sim.integrate(sim_time+t)
		for pl in range(numpl):
			l[pl,ii] = sim.particles[pl+1].l
			po[pl,ii] = sim.particles[pl+1].pomega
			p[pl,ii] = sim.particles[pl+1].P
			e[pl,ii] = sim.particles[pl+1].e

	ps=sim.particles

	maxe(ps,numpl)

	allper=np.array([ps[pl+1].P for pl in range(numpl)])
	res_ang = np.zeros([(numpl*2-3),Nout])
	phi_bs=np.array([])
	if incb: phi_bs = np.zeros([(numpl-1),Nout])
	ang_names = []
	b_names=[]
	thbr_names = ['lap1','lap2','lap3','lap4','lap5']
	twbr_names = ['phi12','phi23','phi34','phi45','phi56','phi67']

	res = nearestRes(allper)

	for kk in range(numpl-1):
		phi = res[kk,0]*l[(kk+1),:] + res[kk,1]*l[kk,:] + res[kk,2]*po[kk,:]
		phib = res[kk,0]*l[(kk+1),:] + res[kk,1]*l[kk,:] + res[kk,2]*po[(kk+1),:]
		res_ang[kk,:] = wrap180(phi)
		if incb: phi_bs[kk,:] = wrap180(phib)
		ang_names = ang_names + [twbr_names[kk]]
		b_names+=[twbr_names[kk]+'b']
	for kk in range(numpl-2):
		lap = res[kk+numpl-1,0]*l[(kk+2),:] + res[kk+numpl-1,1]*l[(kk+1),:] + res[kk+numpl-1,2]*l[kk,:]
		res_ang[(kk+numpl-1),:] = wrap(lap)
		ang_names = ang_names + [thbr_names[kk]]

	time = (sim_time+int_time)/365.25

	numres = np.shape(res_ang)[0]+len(phi_bs)
	fig,axs = plt.subplots(numres,sharex=True,figsize=(8,12))
	fig.subplots_adjust(wspace=0, hspace=0,left=0.2,bottom=0.11,top=0.95)
	i = 0

	if incb:
		res_ang = np.concatenate(res_ang,phi_bs)
		ang_names +=b_names

	for ax in axs:
		#ax.plot(time,res_ang[i,:], mycol[i+1],marker='.',ls='')
		ax.plot(time,res_ang[i,:], 'k',marker='.',ls='')
		ax.set_ylabel(ang_names[i],weight='bold')
		if addlines:
			ax.plot(time,np.ones(len(time))*np.median(res_ang[i,:]),'b-',lw=2)
			ax.plot(time,np.ones(len(time))*(np.median(res_ang[i,:])+2*np.std(res_ang[i,:])),'b--',lw=1)
			ax.plot(time,np.ones(len(time))*(np.median(res_ang[i,:])-2*np.std(res_ang[i,:])),'b--',lw=1)
		i=i+1
	axs[-1].set_xlabel('Time (yr)',weight='bold')

	for ax in axs:
		ax.set_yticks([-180,0,180])
		ax.set_yticks(np.linspace(-180,180,9),minor=True)
		ax.set_yticklabels(['']*9,minor=True)
		ax.set(ylim=[-180,180])
		ax.label_outer()

	for ii in range(numpl-2):
		axs[numpl-1+ii].set_yticks([0,180,360])
		axs[numpl-1+ii].set_yticks(np.linspace(0,360,9),minor=True)
		axs[numpl-1+ii].set_yticklabels(['']*9,minor=True)
		axs[numpl-1+ii].set(ylim=[0,360])

	if incb:
		for ii in range(numpl-1):
			axs[2*numpl-3+ii].set_yticks([0,180,360])
			axs[2*numpl-3+ii].set_yticks(np.linspace(0,360,9),minor=True)
			axs[2*numpl-3+ii].set_yticklabels(['']*9,minor=True)
			axs[2*numpl-3+ii].set(ylim=[0,360])

	pp = PdfPages(savename)
	pp.savefig(fig)
	pp.close()
	plt.close('all')
	plt.cla()
	plt.clf()
	print("plot saved as %s" %savename)
